//QSoft MCI sample
//http://qsoft.cjb.net
//Do not forget to link "winmm.lib"!

#include <windows.h>
#include <mmsystem.h>
#include "resource.h"

TCHAR strFileName[MAX_PATH] = TEXT(""); // The MIDI file name.
UINT MIDI_Device = NULL; // This is the "handle" to the MIDI player.
BOOL Paused = false; // This flag will tell us if we have to Pause or Continue the MIDI.
HWND g_hDlg = NULL; // Global handle to dialog;

void MIDI_Free()
{
 // We tell the MCI to unload the MIDI and stops playing.
 if (MIDI_Device) mciSendCommand(MIDI_Device, MCI_CLOSE, 0, NULL); // Closing the MIDI.
 MIDI_Device = NULL;
}

void OnResPas() // Resume or Pause
{
	MCI_GENERIC_PARMS mciGenericParams; // This hold the handle to window who receives the notify messages.
	Paused = !Paused;
	if (Paused)
	{
		SetDlgItemText(g_hDlg, IDC_STAT, "MIDI is paused!");
		SetDlgItemText(g_hDlg, IDC_RESPAS, "Resume");
		mciSendCommand(MIDI_Device, MCI_PAUSE, MCI_NOTIFY, (DWORD)&mciGenericParams);
	}
	else
	{
		SetDlgItemText(g_hDlg, IDC_STAT, "MIDI is resumed!");
		SetDlgItemText(g_hDlg, IDC_RESPAS, "Pause");
		mciSendCommand(MIDI_Device, MCI_RESUME, MCI_NOTIFY, (DWORD)&mciGenericParams);
	}
}

void OnStop()
{
	if (MIDI_Device) // Means we loaded a MIDI already.
	{
		MCI_GENERIC_PARMS mciGenericParams; // This hold the handle to window who receives the notify messages.
		Paused = true; // I set this to true, because OnResPas will switch it to FALSE(which I want to set the button to "contonue").
		OnResPas();
	 mciSendCommand(MIDI_Device, MCI_SEEK, MCI_NOTIFY | MCI_SEEK_TO_START, (DWORD)&mciGenericParams);
		SetDlgItemText(g_hDlg, IDC_STAT, "MIDI is stopped!");
		// When we stop the MIDI playing, we actually stop and tell the player to start the MID from the start.
	}
	EnableWindow(GetDlgItem(g_hDlg, IDC_STOP), 0);
	EnableWindow(GetDlgItem(g_hDlg, IDC_RESPAS), 0);
}

void OnBrowse()
{
 OPENFILENAME ofn = {sizeof(OPENFILENAME), NULL, NULL,
                     TEXT("MIDI Files\0*.mid;*.midi\0All Files\0*.*\0\0"), NULL,
                     0, 1, strFileName, MAX_PATH, NULL, 0, NULL,
                     TEXT("Open MIDI File"),
                     OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, 0, 0,
                     TEXT(".MID"), 0, NULL, NULL};
	if (GetOpenFileName(&ofn) == TRUE)
	{
		if (MIDI_Device)	OnStop();
		EnableWindow(GetDlgItem(g_hDlg, IDC_PLAY), 1);
		SetDlgItemText(g_hDlg, IDC_STAT, "MIDI is loaded!");
	}
}

void OnPlay()
{
	EnableWindow(GetDlgItem(g_hDlg, IDC_STOP), 1);
	EnableWindow(GetDlgItem(g_hDlg, IDC_RESPAS), 1);
	// We check if the user stopped the MIDI or we just first loading it.
	MCI_PLAY_PARMS mciPlayParms;
 mciPlayParms.dwCallback = (DWORD)g_hDlg; // We pass the handle to dialog because when the MIDI will be finished playing it will notify us.
	if (!MIDI_Device)
	{
  // We tell the MCI to load a MIDI off a file and to start play it once.
  MCI_OPEN_PARMS mciOpenParms;
  mciOpenParms.lpstrDeviceType = "sequencer"; // Means we want to play a MIDI.
  mciOpenParms.lpstrElementName = strFileName; // The MIDI file name.
  mciSendCommand(NULL, MCI_OPEN, MCI_OPEN_TYPE | MCI_OPEN_ELEMENT, (DWORD)&mciOpenParms); //We open the MIDI for playing.
  MIDI_Device = mciOpenParms.wDeviceID; // Getting the Device ID which will play the MIDI.
  mciSendCommand(MIDI_Device, MCI_PLAY, MCI_NOTIFY, (DWORD)&mciPlayParms); // Start play and notify when done.
	}
	else mciSendCommand(MIDI_Device, MCI_PLAY, MCI_NOTIFY, (DWORD)&mciPlayParms);
	SetDlgItemText(g_hDlg, IDC_STAT, "MIDI is played!");
}

BOOL CALLBACK MCIProc(HWND hDlg, UINT MSG, WPARAM wParam, LPARAM lParam)
{
	switch(MSG)
	{
	 case WM_INITDIALOG:
			g_hDlg = hDlg;
			SetDlgItemText(hDlg, IDC_STAT, "Load a MIDI!");
		break;
		case WM_COMMAND:
   switch(LOWORD(wParam))
			{
			 case IDC_BROWSE: OnBrowse();	break;
				case IDC_PLAY: OnPlay(); break;
				case IDC_RESPAS: OnResPas(); break;
				case IDC_STOP: OnStop(); break;
				case IDC_EXIT: EndDialog(hDlg, 0); break;
			}
		break;
	}
	return false;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG), NULL, MCIProc);
	MIDI_Free();
	return true;
}